@@ -1,5 +1,5 @@
 #!/usr/local/bin/perl
-#configure-2.2.15-lvs.pl v0.6 (C) Joseph Mack 1999-2000 jmack@wm7d.net
+#configure-2.2.17-lvs.pl v0.7 (C) Joseph Mack 1999-2000 jmack@wm7d.net
 #
 #released under GPL
 #This code is part of the Linux Virtual Server (LVS) project
@@ -19,6 +19,10 @@
 #            The code no longer can report errors by input line, as the code has read to the end
 #            of the input file before it starts to parse.
 #            added rc.horms, rc.ratz
+#v0.7 Nov 00 bug fixes, mon scripts now work again
+#            create an ntp.conf file for realservers and director
+#            autodetects location of ipchains, route, netstat etc 
+#            assumes ipvsadm is in /sbin
 #
 #Caveat - not all inputs have been tested. 
 #Please send bug reports to Joe jmack@wm7d.net
@@ -90,7 +94,7 @@ The director is a Linux box (kernel 2.4 and
 2.2 supported, 2.0 is no longer supported).
 
 The realservers can have almost any OS (see HOWTO). 
-Realservers running non-Linux unices are configured by code 
+realservers running non-Linux unices are configured by code 
 contributed by Ratz <ratz@tac.ch>. 
 This code has not been well tested.
 Any updates, queries etc, please contact Ratz or myself.
@@ -163,7 +167,7 @@ using ipvsadm on director
 
 =head2 sets IPs and network on realserver(s) 
 
-including non-arp'ing VIP. Realservers can accept packets on VIP
+including non-arp'ing VIP. realservers can accept packets on VIP
 by a network device (eg eth0 or lo:0) device or by transparent proxy. 
 sets default gw
 
@@ -182,6 +186,27 @@ put them into the LVS. Failure of a service on a realserver
 will be detected and this service will be removed from the LVS
 list of offered services, till the service comes back on line.
 
+b<configure_lvs> generates an ntp.conf file for the director 
+and real-servers
+
+=head2 sets up ntp conf files 
+
+These files can be copied to /etc/ntp.conf
+on all machines. tested with ntp-4.0.99g. ntp code from
+http://www.ntp.org
+
+The files configure the director as a stratum 3 time server and the
+real-servers as stratum 4 time servers. Machines peer with each other
+and real-servers being a lower priority stratum will defer to the
+director eventually becoming synchronised with the
+directors CMOS clock. To synchronise the director with an external
+ntp server, look up the docs on the ntp site for a public time server
+near you on the net and uncomment the line in the director's ntp.conf
+(it has a commented out public ntp server in North America).
+
+The machines will take a long time to synchronise if they are more than
+a minute different in time (this is part of the ntp protocol).
+
 =head1 WHICH IS THE DIRECTOR?
 
 answer: For the moment the director is the one with the 
@@ -312,21 +337,24 @@ script to work with directors and realservers which have 1,2..n NICs.
 Here are the variables required, with examples.
 
 	#LVS_TYPE: VS_TUN|VS_NAT|VS_DR 
-			#(string is matched with tun/nat/dr, 
-			# any string containing tun, nat or dr will do. eg tunnel, natural, drive)
+	#(string is matched with tun/nat/dr, 
+	# any string containing tun, nat or dr will do. eg tunnel, natural, drive)
 LVS_TYPE=VS_DR
 
 
 	#INITIAL_STATE: on|off  
-	#Only useful if you are using mon, otherwise ignore 
-	#(leave "on" or ommit, or comment out)
+	#Only useful if you are using mon 
+	#(or some tool to determine the state of servers and services)
+	#otherwise ignore (leave "on" or ommit, or comment out)
 	#
-	#suggested: for testing - "on", for production systems - "off", 
+	#suggested: for testing - "on"
+	#           for production systems - "off", 
 	#if you don't understand the following then choose "on"
 	#
-	#when testing LVS, the services (eg http, telnet) are already running 
-	#on the real servers, before you start adding servers:services 
-	#with ipvsadm on the director, and before you make a request from the client, 
+	#when testing an LVS, you normally have the services (eg http, telnet) 
+	#already running on the real servers, before you run ipvsadm 
+	#(to add servers and services) on the director,
+	#and before you make a request from the client, 
 	#
 	#In a production system, if ipvsadm adds servers:services to
 	#the director as soon as the director boots, the servers 
@@ -340,8 +368,21 @@ LVS_TYPE=VS_DR
 	#detect an on-line server and add entries to the LVS
 	#table by running ipvsadm, bringing the server into the LVS.
 	#
-	#since I don't have a big system to test this on, can someone let
-	#me know if this actually works for big systems. - thanks Joe
+	#If you have INITIAL_STATE off, then the configure script
+	#will setup the LVS with no entries, and allow mon to add the
+	#entries as the services come up on the real-servers.
+	#
+	#If you choose "off" here, mon must be (re)started after 
+	#running the rc.lvs scripts. This is incase the real-servers
+	#are already up. Mon upalerts only occur on a transition 
+	#of the services from down to up. If mon is already running 
+	#and the real-servers are offering services and the rc.lvs 
+	#script sets up an LVS with services off, then the LVS will
+	#just sit there in the off state. You have to re-HUP
+	#mon (to get it to re-read its config file and test the services).
+	#
+	#I've only got mon-3.71 to work. The 3.8 series have problems for
+	#me and Jim Trocki, the author of mon doesn't reply to e-mails.
 	#
 INITIAL_STATE=on  
 
@@ -498,7 +539,14 @@ lvs_nat.conf
 
 =head1 BUGS
 
-I haven't even thought about looking for bugs yet.
+Someone has reported a bug setting up VS-Tun. Haven't looked into it yet.
+
+=head1 TODO
+
+merge configure-lvs with HA scripts (eg Horms stuff)
+Have script setup router (if it runs sh that is).
+Not setup default route on director for VS-DR (it doesn't need one)
+Have default route setup automatically for VS-NAT real-servers.
 
 =head1 HISTORY
 
@@ -517,9 +565,9 @@ Entries in conf file can be in any order
 (this is not particularly useful) and (more importantly)
 decisions can now be made using two (or more) of input data. 
 
-=head1 TODO
-
-1. merge configure-lvs with HA scripts (eg Horms stuff)
+B<v0.7 Dec 99>
+Bug fixes and make script compatible with LVS-mini-HOWTO.
+add creation of ntp.conf files.
 
 =head1 THANKS TO
 
@@ -551,7 +599,7 @@ use Net::DNS;
 use Socket;
 $| = 1; #flush buffers
 
-my ($lvs_type, $connection_parameter, $connection_type);
+my ($lvs_type, $connection_type, $connection_parameter);
 my $initial_state;
 my $line; #input line
 
@@ -561,6 +609,8 @@ my $director_file = "rc.lvs_${lvs_type}director";
 my $server_file = "rc.lvs_${lvs_type}server";
 my $rc_file = "rc.lvs_$lvs_type";
 my $mon_file = "mon_lvs${lvs_type}.cf";
+my $director_ntp_file;
+my $realserver_ntp_file;
 my @params;
 my ($vdevice,$vip, $vnetmask, $vbroadcast); #VIP setup for VS_TUN, VS_DR
 my ($inside_device, $inside_ip, $inside_network, $inside_netmask, $inside_broadcast,$dot_quad, $classless, $log_2); #server network for VS_NAT
@@ -568,15 +618,18 @@ my ($inside_device, $inside_ip, $inside_network, $inside_netmask, $inside_broadc
 #service setup
 my $default_scheduler = "wlc";
 my ($proto, $service, $scheduler, $t_switch, $timeout, $server, $weight, $weight_lvsadm); 
-my ($service_name, $port, $protocol, $server_name,$server_ip);
+my ($service_name, $port, $protocol, $server_name, $server_ip);
 my ($server_port, $server_ipport, $server_service_name);
 my (@service_lines, $element, $line_reference); #the input lines containing SERVICES, each element of the line, and the reference to the line inside @service_lines
 my (@redirect_lines, @services);#for running rc.horms
 
+#director setup
+my $director_inside_ip_name;
 
 #server setup
 my ($rs_vip_device, $snetdevice, $server_counter, $monitor); 
-my @servers; #list of servers
+my @servers; #list of server IPs
+my @server_names; #list of server names
 
 #kernel version
 my $uname_r_known = "N";
@@ -601,7 +654,7 @@ my @schedulers = ("rr","wrr","lc","wlc","pcc");
 #list kernels that this script is thought to work on
 #pre kernels are named like 2.2.15pre5. 
 #this will be rev'ed down to 2.2.14.
-my @uname_r_ok = ("2.2.10","2.2.12","2.2.13","2.2.14","2.2.15","2.2.16","2.4.0");
+my @uname_r_ok = ("2.2.10","2.2.12","2.2.13","2.2.14","2.2.15","2.2.16","2.2.17","2.4.0");
 my $uname_r;
 
 #------------------------------------------------------------
@@ -613,39 +666,82 @@ my $initialise_variables;
 @LVS_USERS="lvs-users@linuxvirtualserver.org"
 @UNAME=`uname`
 @UNAME_R=`uname -r`
-@#initialise utilities. some of them will be in the path hopefully. 
-@#At least this way a "file not found" error will result if they're in a different place.
-@#Otherwise the "" command would have been issued with lots of arguements.
-@#
+@#initialise utilities. 
+@#You could hope they'd all be in the path, but I'm just going to find them anyhow
 @
-@IFCONFIG="ifconfig"
-@NETSTAT="netstat"
-@ROUTE="route"
-@AWK="awk"
-@
-@if [ `uname` = "SunOS" ]
-@then
-@	{
-@	IFCONFIG="/usr/sbin/ifconfig"
-@	NETSTAT="/usr/bin/netstat"
-@	ROUTE="/usr/sbin/route"
-@	AWK="/usr/bin/awk"
-@	#echo "Solaris $IFCONFIG $NETSTAT $ROUTE"
-@	}
-@fi
-@if [ `uname` = "Linux" ]
-@then
-@	{
-@	IFCONFIG="/sbin/ifconfig"
-@	NETSTAT="/bin/netstat"
-@	ROUTE="/sbin/route"
-@	AWK="/usr/bin/awk"
-@	#echo "Linux $IFCONFIG $NETSTAT $ROUTE"
-@	}
-@fi
+@#ipvsadm is going to have to be in /sbin
 @IPVSADM="/sbin/ipvsadm"
-@IPCHAINS="/sbin/ipchains"
-@FPING="/usr/local/bin/fping"
+@
+@echo "looking for standard utilities"
+@CMDS="ipchains fping ping ifconfig netstat route awk" 
+@DIRECTORIES="/usr/local/bin /usr/bin /usr/sbin /bin /sbin"
+@
+@for CMD in $CMDS
+@do
+@	for DIRECTORY in $DIRECTORIES
+@	do
+@		if [ -x ${DIRECTORY}/$CMD ]
+@		then
+@			case "$CMD"
+@			in
+@				ipchains)	#echo "ipchains"
+@						IPCHAINS=${DIRECTORY}/$CMD
+@						echo "\$IPCHAINS=${IPCHAINS}"
+@						;;
+@				fping) 		#echo "fping"
+@						FPING=${DIRECTORY}/$CMD
+@						echo "\$FPING=${FPING}"
+@						;;
+@				ping) 		#echo "ping"
+@						PING="${DIRECTORY}/$CMD -c 1"
+@						echo "\$PING=${PING}"
+@						;;
+@				ifconfig)	#echo "ifconfig"
+@						IFCONFIG=${DIRECTORY}/$CMD
+@						echo "\$IFCONFIG=${IFCONFIG}"
+@						;;
+@				netstat) 	#echo "netstat"
+@						NETSTAT=${DIRECTORY}/$CMD
+@						echo "\$NETSTAT=${NETSTAT}"
+@						;;
+@				route) 		#echo "route"
+@						ROUTE=${DIRECTORY}/$CMD
+@						echo "\$ROUTE=${ROUTE}"
+@						;;
+@				awk)		#echo "awk"
+@						AWK=${DIRECTORY}/$CMD
+@						echo "\$AWK=${AWK}"
+@						;;
+@			esac
+@		fi
+@	done
+@done
+@
+@#fping is a version of ping designed for mon 
+@#which returns the string "foo is alive" and sets $? = 0
+@#it only sends one ping. 
+@#here, we're only using the single ping feature and we aren't looking for the returned string
+@#if fping is not found, test if ping will do
+@#
+@#see if FPING set
+@if [ "$FPING" = "" ]
+@then
+@	#FPING not found
+@	$PING localhost >/dev/null 2>&1
+@	if [ $? = "0" ]
+@	then
+@		echo "fping not found, using ping instead"
+@		FPING=$PING
+@		echo "\$FPING=$FPING"
+@
+@	else
+@		echo "fping not found, ping -c 1 fails"
+@		echo "don't have a substitute for fping, exiting"
+@		exit 0
+@	fi 
+@fi
+@echo "done"
+@#--------------------------- 
 INITIAL
 
 my $ratz;
@@ -705,6 +801,7 @@ my $ratz;
 @
 @detect_realserver_os()
 @	{
+@	#despite the name of this routine, it installs the VIP for known OS's 
 @	DEV=$SERVER_VIP_DEVICE
 @	SHORT_DEV=${DEV%:*}	#lo:0 -> lo, tunl0 -> tunl0
 @	#echo "SHORT_DEV=$SHORT_DEV"
@@ -1065,7 +1162,6 @@ my $general_boilerplate;
 @	fi
 @	}
 @
-@#--------end boilerplate------------------------------------
 @
 @check_for_DIIP(){
 @	#since this is a realserver, there should be no DIIP here 
@@ -1104,14 +1200,20 @@ my $general_boilerplate;
 @	 	echo "bypassing ping check of VIP $VIP" 
 @	else
 @	 	echo "director is accepting packets on network device $DIRECTOR_VIP_DEVICE" 
-@		echo "pinging VIP " 
+@		echo "VIP not on real-server at this stage" 
+@		echo "VIP will be on director" 
+@		echo "pinging VIP" 
 @		$FPING $VIP	
 @		if [ "$?" = "0" ] 
 @		then 
-@			echo "$VIP found, good. It's not on this server, assume it's on the director. "
+@			echo "$VIP found, good. It's not on this real-server, assume it's on the director. "
 @		else 
-@			echo "warning:$VIP not found, presumably director has not been set up yet"
-@			echo "rerun this script after setting up the director if you want all checks to be positive"
+@			echo "warning:$VIP not pingable."
+@			echo "depending on the default gw on the real-server, and the network of the VIP,"
+@			echo "you may or may not be able to ping the VIP on the director from the real-server."
+@			echo "I'll work on rules to give a pass/fail here."
+@			echo "Untill I do, note this result in case the LVS doesn't work."
+@			echo "If you didn't run the script on the director first, then go do this now."
 @		fi 
 @	fi
 @	} #check_for_vip_on_director 
@@ -1352,20 +1454,23 @@ sub write_mon_hostgroup
 		}
 	print MON_FILE "	allow_empty_group \n";
 	print MON_FILE "	period wd {Sun-Sat} \n";
-	print MON_FILE "		alert mail.alert root \n";
-	print MON_FILE "		upalert mail.alert root \n";
+	print MON_FILE "	#alertevery 1h \n";
+	print MON_FILE "		#alert mail.alert root \n";
+	print MON_FILE "		#upalert mail.alert root \n";
 	print MON_FILE "		#-V is virtual service, -R is remote service, -P is protocol, -A is add/delete (t|u)\n";
 	if ($lvs_type =~ /tun|dr/)
 		{
 		print MON_FILE "		alert virtualserver.alert -A -d -P -$proto -V $vip:$port -R $server_ip \n";
-		#print MON_FILE "		upalert virtualserver.alert -A -a -P -$proto -V $vip:$port -R $server_ip -T $connection_parameter -w $weight\n";
-		print MON_FILE "		upalert virtualserver.alert -A -a -P -$proto -V $vip:$port -R $server_ip -T \$CONNECTION_TYPE -w $weight\n";
+		#print MON_FILE "		upalert virtualserver.alert -A -a -P -$proto -V $vip:$port -R $server_ip -T $connection_type -w $weight\n";
+		#print MON_FILE "		upalert virtualserver.alert -A -a -P -$proto -V $vip:$port -R $server_ip -T \$CONNECTION_TYPE -w $weight\n";
+		print MON_FILE "		upalert virtualserver.alert -A -a -P -$proto -V $vip:$port -R $server_ip -T $connection_parameter -w $weight\n";
 		}
 	elsif ($lvs_type eq "nat")
 		{
 		print MON_FILE "		alert virtualserver.alert -A -d -P -$proto -V $inside_ip:$port -R $server_ip:{$server_port} \n";
-		#print MON_FILE "		upalert virtualserver.alert -A -a -P -$proto -V $inside_ip:$port -R ${server_ip}:${server_port} -T $connection_parameter -w $weight\n";
-		print MON_FILE "		upalert virtualserver.alert -A -a -P -$proto -V $inside_ip:$port -R ${server_ip}:${server_port} -T \$CONNECTION_TYPE -w $weight\n";
+		#print MON_FILE "		upalert virtualserver.alert -A -a -P -$proto -V $inside_ip:$port -R ${server_ip}:${server_port} -T $connection_type -w $weight\n";
+		#print MON_FILE "		upalert virtualserver.alert -A -a -P -$proto -V $inside_ip:$port -R ${server_ip}:${server_port} -T \$CONNECTION_TYPE -w $weight\n";
+		print MON_FILE "		upalert virtualserver.alert -A -a -P -$proto -V $inside_ip:$port -R ${server_ip}:${server_port} -T $connection_parameter -w $weight\n";
 		}
 	else	
 		{
@@ -1380,9 +1485,13 @@ sub write_mon_hostgroup
 sub open_files
 	{
 	$rc_file = "rc.lvs_$lvs_type";
-	$mon_file = "mon_lvs${lvs_type}.cf";
 	open (RC_FILE, ">$rc_file")|| die "Can't write to $rc_file $!\n";
+	$mon_file = "mon_lvs${lvs_type}.cf";
 	open (MON_FILE, ">$mon_file")|| die "Can't write to $mon_file $!\n";
+	$director_ntp_file = "ntp.conf.director";
+	open (DIRECTOR_NTP_FILE, ">$director_ntp_file")|| die "Can't write to $director_ntp_file $!\n";
+	$realserver_ntp_file = "ntp.conf.realserver";
+	open (REALSERVER_NTP_FILE, ">$realserver_ntp_file")|| die "Can't write to $realserver_ntp_file $!\n";
 	}
 
 sub print_headers
@@ -1568,7 +1677,7 @@ sub setup_ipvsadm_table
 
 sub setup_director_insideip
 	{
-	my $director_inside_ip_name;
+	#hmm, this isn't called anymore
 	($director_inside_ip_name, $inside_ip) = parse_hostname($inside_ip);
 	print RC_FILE "	#setup DIRECTOR_INSIDEIP\n";
 	#print RC_FILE "	echo \"adding ethernet device and routing for DIRECTOR_INSIDEIP \$DIRECTOR_INSIDEIP \"\n";
@@ -1652,7 +1761,7 @@ sub install_realserver_vip_device
 		print RC_FILE "	NEW_VIP_DEVICE=\$SERVER_VIP_DEVICE \n";
 		print RC_FILE "	remove_vip \n";
 		print RC_FILE "	NEW_VIP_DEVICE=\"\" \n";
-		print RC_FILE "	detect_realserver_os \n";
+		print RC_FILE "	detect_realserver_os #installs VIP\n";
 		print RC_FILE "	echo \" \"\n";
 		}
 
@@ -1738,6 +1847,11 @@ sub check_default_route_nat
 sub print_open_realserver
 	{
 	print RC_FILE "setup_realserver() {\n";
+	#moved here Dec 2000
+	#original location had real-server pinging VIP etc, 
+	#with whatever default gw was installed at the time. 
+	#This could be the original rc.inet1, or none or something else.
+	install_realserver_gw;
 	print RC_FILE "	echo \"LVS realserver type \$LVS_TYPE \"\n";
 	print RC_FILE "	#don't bother removing device with VIP if it's going to be the replacement \n";
 	print RC_FILE "	NEW_VIP_DEVICE=\$SERVER_VIP_DEVICE \n";
@@ -1798,7 +1912,8 @@ sub print_open_realserver
 		print RC_FILE "	NEW_VIP_DEVICE=\"\" \n";
 		check_default_route_nat
 		}
-	install_realserver_gw;
+	#orig location
+	#install_realserver_gw;
 
 	}#sub print_open_realserver
 
@@ -2113,6 +2228,8 @@ sub print_service_lines
 		($server_name,$server_ip)=parse_hostname($server_ip);
 		#add IP of server to @servers
 		push (@servers, $server_ip);
+		#add server names to @server_names
+		push (@server_names, $server_name);
 
 		#accumulate the number of servers out there
 		#use later as a test to see if I'm the director or the server
@@ -2197,6 +2314,45 @@ sub install_print_ipvsadm_lines
 	print RC_FILE  "\n";
 	} #install_print_ipvsadm_lines
 
+sub print_director_ntp_file
+	{
+	print DIRECTOR_NTP_FILE "#director /etc/ntp.conf \n";
+	print DIRECTOR_NTP_FILE "#pick (uncomment) a server near you (this one is in eastern USA) \n";
+	print DIRECTOR_NTP_FILE "#server louie.udel.edu \n";
+	print DIRECTOR_NTP_FILE "driftfile /etc/ntp.drift \n";
+	print DIRECTOR_NTP_FILE "logfile /var/adm/ntplog \n";
+	print DIRECTOR_NTP_FILE "disable auth \n";
+	print DIRECTOR_NTP_FILE "server 127.127.1.0      	#local CMOS clock \n";
+	print DIRECTOR_NTP_FILE "#tell everyone we're a stratum 3 ntp server \n";
+	print DIRECTOR_NTP_FILE "fudge 127.127.1.0 stratum 3 \n";
+	}
+
+sub print_realserver_ntp_file
+	{
+	print REALSERVER_NTP_FILE "#realserver /etc/ntp.conf \n";
+	print REALSERVER_NTP_FILE "driftfile /etc/ntp.drift \n";
+	print REALSERVER_NTP_FILE "logfile /var/adm/ntplog \n";
+	print REALSERVER_NTP_FILE "disable auth \n";
+	print REALSERVER_NTP_FILE "server 127.127.1.0 #local clock \n";
+	print REALSERVER_NTP_FILE "#tell everyone we're a stratum 4 ntp server \n";
+	print REALSERVER_NTP_FILE "#we'll defer to the director which is stratum 3 \n";
+	print REALSERVER_NTP_FILE "fudge 127.127.1.0 stratum 4 \n";
+	($director_inside_ip_name, $inside_ip) = parse_hostname($inside_ip);
+	print REALSERVER_NTP_FILE "server $director_inside_ip_name \n";
+	while ($server_name = $server_names[0])	#$server_name may be an IP
+		{
+		($server_name,$server_ip) = parse_hostname($server_name);
+		#loop through IPs of known realservers comparing them to the IP of this machine.
+		shift @server_names;
+		#don't test for 127.0.0.1
+		if ($server_name eq "localhost") {next;}
+		#this code will print a $server_name for each of its services
+		#need to fix this sometime
+		print DIRECTOR_NTP_FILE "peer $server_name \n";
+		print REALSERVER_NTP_FILE "peer $server_name \n";
+		}
+	}
+
 sub do_for_director_or_realserver
 	{
 	print RC_FILE "#Determine if I'm a realserver, director (or fail)\n";
@@ -2207,11 +2363,12 @@ sub do_for_director_or_realserver
 	print RC_FILE "#Without the blank, \"192.168.1.1\" matches \"192.168.1.1x\", but \"192.168.1.1 \" will not.\n";
 	print RC_FILE "#a similar problem exists at the front of the IP, but there's no easy way to handle it.\n";
 	print RC_FILE "#In linux the previous char is a blank or \'\', in solaris it's \'#\' \n";
-	print RC_FILE "#If I'm not a server, then maybe I'm the director. If I'm the director, then ipvsadm will be on this machine.\n";
+	print RC_FILE "#If I'm not a server, then maybe I'm the director.\n";
+	print RC_FILE "#If I'm the director, then ipvsadm will be on this machine.\n";
 	print RC_FILE "#If this fails, give up, exit and cry for help\n";
 	print RC_FILE "\n";
 	print RC_FILE "if \\\n";
-	while ($server_ip = $servers[0])	#$erver_ip may be a $server_name
+	while ($server_ip = $servers[0])	#$server_ip may be a $server_name
 		{
 		($server_name,$server_ip) = parse_hostname($server_ip);
 		#loop through IPs of known realservers comparing them to the IP of this machine.
@@ -2247,6 +2404,8 @@ sub do_for_director_or_realserver
 	print RC_FILE "	then\n";
 	print RC_FILE "		{\n";
 	print RC_FILE "		setup_director\n";
+	#print DIRECTOR_NTP_FILE "$director_ntp_conf_txt \n";
+	print_director_ntp_file;
 	print RC_FILE "		}\n";
 	print RC_FILE "	else\n";
 	print RC_FILE "		{\n";
@@ -2263,6 +2422,8 @@ sub do_for_director_or_realserver
 	print RC_FILE "else \n";
 	print RC_FILE "	{\n";
 	print RC_FILE "	setup_realserver\n";
+	#print REALSERVER_NTP_FILE "$realserver_ntp_conf_txt \n";
+	print_realserver_ntp_file;
 	print RC_FILE "	}\n";
 	print RC_FILE "fi\n";
 	}
@@ -2277,11 +2438,12 @@ sub write_main
 	print RC_FILE "DIRECTOR_INSIDEIP=$inside_ip \n";
 	print RC_FILE "DIRECTOR_DEFAULT_GW=$director_gw \n";
 	print RC_FILE "LVS_TYPE=vs-$lvs_type \n";
+#	print RC_FILE "REALSERVERS=\"";
 	print RC_FILE "HIDDEN=\"Y\" \n";
 	print RC_FILE "NAT=\"-m\" \n";
 	print RC_FILE "DR=\"-g\" \n";
 	print RC_FILE "TUN=\"-i\" \n";
-	print RC_FILE "CONNECTION_TYPE=\"$connection_parameter\" \n";
+	print RC_FILE "CONNECTION_TYPE=\"$connection_type\" \n";
 	print RC_FILE "#echo \"connection type = \$CONNECTION_TYPE\" \n";
 	if ($lvs_type eq "nat")
 		{
@@ -2348,17 +2510,20 @@ sub read_conf_file
 			if (uc($params[0]) =~/TUN/) 
 				{ 
 				$lvs_type = "tun"; 
-				$connection_parameter= '$TUN';
+				$connection_type= '$TUN';
+				$connection_parameter='-i';
 				}
 			if (uc($params[0]) =~/NAT/) 
 				{ 
 				$lvs_type = "nat"; 
-				$connection_parameter= '$NAT';
+				$connection_type= '$NAT';
+				$connection_parameter= '-m';
 				}
 			if (uc($params[0]) =~/DR/) 
 				{ 
 				$lvs_type = "dr"; 
-				$connection_parameter= '$DR';
+				$connection_type= '$DR';
+				$connection_parameter= '-g';
 				}
 			($lvs_type =~ /tun|nat|dr/) || die "unknown LVS_TYPE $lvs_type ";
 			#print "LVS_TYPE: $lvs_type \n";
@@ -2561,10 +2726,6 @@ else	{
 	}
 setup_ipvsadm_table;
 setup_director_masq_table;
-#currently setting this up on machine known to be director
-#eventually will have this already setup and will use this IP
-#to deteect whether are director or not
-#setup_director_insideip;
 
 print_close_setup_director;
 print_close_mon_file;
@@ -2583,4 +2744,6 @@ my @cannot;
 @cannot = grep {not chmod 755, $_} $rc_file;
 die "$0: could not chmod @cannot\n" if @cannot;
 
-#-----configure_lvs-2.2.15.pl---------------------------------
+close DIRECTOR_NTP_FILE;
+close REALSERVER_NTP_FILE;
+#-----configure_lvs-2.2.16.pl---------------------------------
